/*
 * File         : Message.h
 * Author       : Tomasz Stachowiak
 * Created      : Wed May 20 18:35:14 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#ifndef __MESSAGE_H
#define __MESSAGE_H

#include <BVN/BVN.h>

template <class MSG_HEADER, class PEER_STREAM>
class Message
{
 public:
  Message() { body_ = 0; };
  Message(MSG_HEADER &header) : header_(header) { body_ = 0; };
  virtual ~Message();
  virtual int recv(PEER_STREAM *stream);
  virtual int send(PEER_STREAM *stream);
  virtual int execute()=0;
  virtual int destroy();
  virtual void set(char *body, int size);

 protected:
  
  MSG_HEADER header_;
  char *body_;
};

#define MH MSG_HEADER
#define PS PEER_STREAM

template <class MH, class PS>
Message<MH, PS>::~Message()
{
}

template <class MH, class PS>
int Message<MH, PS>::recv(PEER_STREAM *stream)
{
  int size;
  int n;

  BVN_DEBUG("Message::recv\n");

  n = header_.recv(stream);
  if(n == -1)
    {
      BVN_ERROR("Message::recv\n");
      return -1;
    }
  if(n == 0)
    return 0;

  size = header_.get_size();
  BVN_DEBUG("Message::recv, body size = %d\n", size);
  if(size > 0)
    {
      body_ = new char[size];
      n = stream->recv_n(body_, size);
      if(n == -1)
	{
	  BVN_ERROR("Message::recv\n");
	  return -1;
	}
      if(n == 0)
	return 0;
    }
  
  return 1;
}

template <class MH, class PS>
int Message<MH, PS>::send(PEER_STREAM *stream)
{
  int size;
  int n;

  n = header_.send(stream);
  if(n == -1)
    {
      BVN_ERROR("Message::send\n");
      return -1;
    }

  size = header_.get_size();

  if(size > 0)
    {

      if(body_ == 0)
	{
	  BVN_ERROR("Message::send\n");
	  return -1;
	}

      if(stream->send_n(body_, size) == -1)
	{
	  BVN_ERROR("Message::send\n");
	  return -1;
	}
      
    }
  
  return 0;
}

template <class MH, class PS>
int Message<MH, PS>::destroy()
{
  if(body_ != 0)
    {
      delete body_;
      body_ = 0;
    }

  return 0;
}


template <class MH, class PS>
void Message<MH, PS>::set(char *body, int size)
{
  if(body_ != 0)
    delete body_;

  body_ = body;
  header_.set_size(size);
}

#endif

